package de.unileipzig.datenschutz.paper10.demo;

import de.unileipzig.datenschutz.paper10.demo.data.FontsFingerprint;
import de.unileipzig.datenschutz.paper10.demo.data.ScreenSizeFinerPrint;
import de.unileipzig.datenschutz.paper10.demo.data.UserSession;
import de.unileipzig.datenschutz.paper10.demo.data.UserSessionRepo;
import de.unileipzig.datenschutz.paper10.demo.dto.FontList;
import de.unileipzig.datenschutz.paper10.demo.dto.FontListDto;
import de.unileipzig.datenschutz.paper10.demo.dto.ScreenSizeData;
import jakarta.servlet.http.HttpSession;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.Optional;

@RestController
public class UserBackController {

    @Autowired
    UserSessionRepo userSessionRepo;

    @PostMapping("/screensize")
    public void receiveScreenSizeData(@RequestBody ScreenSizeData screenSizeData,
                                        final HttpSession session) {
        Optional<UserSession> userSession = userSessionRepo.findBySessionID(session.getId());

        if (userSession.isEmpty()){
            UserSession newSession = new UserSession(session.getId());
            userSession = Optional.of(userSessionRepo.save(newSession));
        }

        if (userSession.get().getScreenSizeFinerPrint() == null){
            ScreenSizeFinerPrint screenSizeFinerPrint = new ScreenSizeFinerPrint(screenSizeData.getScreenSize().getWidth(), screenSizeData.getScreenSize().getHeight());
            userSession.get().setScreenSizeFinerPrint(screenSizeFinerPrint);
            userSessionRepo.save(userSession.get());
        }


    }

    @PostMapping("/fonts")
    public void receiveFontsData(@RequestBody Object object,
                                 final HttpSession session){

        FontList fontList = FontListDto.parseToFontList(object);

        Optional<UserSession> userSession = userSessionRepo.findBySessionID(session.getId());

        if (userSession.isEmpty()){
            UserSession newSession = new UserSession(session.getId());
            userSession = Optional.of(userSessionRepo.save(newSession));
        }

        if (userSession.get().getFontsFingerprint() == null){
            FontsFingerprint fontsFingerprint = new FontsFingerprint(fontList.getInstalledFonts());
            userSession.get().setFontsFingerprint(fontsFingerprint);
            userSessionRepo.save(userSession.get());

        }

    }
}
