package de.unileipzig.datenschutz.paper10.demo;

import de.unileipzig.datenschutz.paper10.demo.data.*;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;

import ua_parser.Parser;
import ua_parser.Client;

import java.util.*;
import java.util.stream.Collectors;

@Controller
public class UserFrontController {

    private static final Parser uaParser = new Parser();


    @Autowired
    UserSessionRepo userSessionRepo;

    @Autowired
    ScreenSizeFinerPrintRepo screenSizeFinerPrintRepo;

    @Autowired
    FontsFingerprintRepo fontsFingerprintRepo;

    @Autowired
    FontsDataRepo fontsDataRepo;
    
    @Autowired
    HeaderFingerprintRepo headerFingerprintRepo;

    @GetMapping("/")
    public String index(final HttpServletRequest request,
                        final HttpSession session,
                        final Model model){

        Optional<UserSession> userSession = userSessionRepo.findBySessionID(session.getId());

        if (userSession.isEmpty()){
            UserSession newSession = new UserSession(session.getId());
            userSession = Optional.of(userSessionRepo.save(newSession));
        }

        if (userSession.get().getHeaderFingerprint() == null){
            userSession.get().setHeaderFingerprint(parseHeader(request));
            userSessionRepo.save(userSession.get());
        }

        addModelData(model, userSession.get());

        return "main";
    }
    private static HeaderFingerprint parseHeader(final HttpServletRequest request){

        Client c = uaParser.parse(request.getHeader("user-agent"));

        return new HeaderFingerprint(
                c.os.family,
                c.os.major,
                c.os.minor,
                c.device.family,
                c.userAgent.family,
                c.userAgent.major,
                c.userAgent.minor,
                Arrays.stream(request.getHeader("accept").split("[,;]")).collect(Collectors.toSet()),
                Arrays.stream(request.getHeader("accept-encoding").split(", ")).collect(Collectors.toSet()),
                Arrays.stream(request.getHeader("accept-language").split("[,;]")).collect(Collectors.toSet())
                );

    }

    private void addModelData(final Model model,
                                     UserSession userSession){

        model.addAttribute("session_id", userSession.getSessionID());

        if (userSession.getScreenSizeFinerPrint() != null){
            model.addAttribute("show_screen_size", true);
            model.addAttribute("screenSizeFinerPrint_width", userSession.getScreenSizeFinerPrint().getWidth());
            model.addAttribute("screenSizeFinerPrint_width_count", screenSizeFinerPrintRepo.findAllByWidthEquals(userSession.getScreenSizeFinerPrint().getWidth()).size()/2-1);
            model.addAttribute("screenSizeFinerPrint_height", userSession.getScreenSizeFinerPrint().getHeight());
            model.addAttribute("screenSizeFinerPrint_height_count", screenSizeFinerPrintRepo.findAllByHeightEquals(userSession.getScreenSizeFinerPrint().getHeight()).size()/2-1);
        }
        if (userSession.getFontsFingerprint() != null) {
            model.addAttribute("show_fonts", true);
            model.addAttribute("fontsFingerprint_installedFonts", new FontsModel().generateFontsModelSet(userSession.getFontsFingerprint().getInstalledFonts()));
        }

        if (userSession.getHeaderFingerprint() != null) {
            model.addAttribute("show_header", true);
            HeaderFingerprint headerFingerprint = userSession.getHeaderFingerprint();
            model.addAttribute("headerFingerprint_os_type", headerFingerprint.getOsType());
            model.addAttribute("headerFingerprint_os_type_count", headerFingerprintRepo.findByOsTypeIs(headerFingerprint.getOsType()).size()-1);
            model.addAttribute("headerFingerprint_os_version", headerFingerprint.getOsVersion());
            model.addAttribute("headerFingerprint_os_version_count", headerFingerprintRepo.findByOsTypeIsAndOsVersionIs(headerFingerprint.getOsType(), headerFingerprint.getOsVersion()).size()-1);
            model.addAttribute("headerFingerprint_os_sub_version", headerFingerprint.getOsSubVersion());
            model.addAttribute("headerFingerprint_os_sub_version_count", headerFingerprintRepo.findByOsTypeIsAndOsVersionIsAndOsSubVersionIs(headerFingerprint.getOsType(), headerFingerprint.getOsVersion(), headerFingerprint.getOsSubVersion()).size()-1);
            model.addAttribute("headerFingerprint_device", headerFingerprint.getDevice());
            model.addAttribute("headerFingerprint_device_count", headerFingerprintRepo.findByDeviceIs(headerFingerprint.getDevice()).size()-1);
            model.addAttribute("headerFingerprint_useragent_family", headerFingerprint.getUseragentFamily());
            model.addAttribute("headerFingerprint_useragent_family_count", headerFingerprintRepo.findByUseragentFamilyIs(headerFingerprint.getUseragentFamily()).size()-1);
            model.addAttribute("headerFingerprint_useragent_family_version", headerFingerprint.getUseragentVerison());
            model.addAttribute("headerFingerprint_useragent_family_version_count", headerFingerprintRepo.findByUseragentFamilyIsAndUseragentVerisonIs(headerFingerprint.getUseragentFamily(), headerFingerprint.getUseragentVerison()).size()-1);
            model.addAttribute("headerFingerprint_useragent_family_sub_version", headerFingerprint.getUseragentSubVersion());
            model.addAttribute("headerFingerprint_useragent_family_sub_version_count", headerFingerprintRepo.findByUseragentFamilyIsAndUseragentVerisonIsAndUseragentSubVersionIs(headerFingerprint.getUseragentFamily(), headerFingerprint.getUseragentVerison(), headerFingerprint.getUseragentSubVersion()).size()-1);
            model.addAttribute("headerFingerprint_accepts", headerFingerprint.getAccepts().toString());
            //model.addAttribute("headerFingerprint_accepts_count", filterAcceptsEquals(headerFingerprint.getAccepts(), headerFingerprintRepo.findAllByAcceptsIn(headerFingerprint.getAccepts())));
            model.addAttribute("headerFingerprint_encoding", headerFingerprint.getEncoding().toString());
            //model.addAttribute("headerFingerprint_encoding_count", filterAcceptsEquals(headerFingerprint.getEncoding(), headerFingerprintRepo.findAllByEncodingIn(headerFingerprint.getEncoding())));
            model.addAttribute("headerFingerprint_languages", headerFingerprint.getLanguages().toString());
            //model.addAttribute("headerFingerprint_languages_count", filterAcceptsEquals(headerFingerprint.getLanguages(), headerFingerprintRepo.findAllByLanguagesIs(headerFingerprint.getLanguages())));
        }
    }


    public class FontsModel {
        private final String name;
        private final int count;

        public FontsModel(){

            name = null;
            count = 0;
        }

        public FontsModel(String name, int count){
            this.name = name;
            this.count = count;
        }

        public ArrayList<FontsModel> generateFontsModelSet(Set<FontsData> usableFonts){
            FontsData[] ordered = usableFonts.toArray(FontsData[]::new);
            Arrays.sort(ordered, Comparator.comparing(FontsData::getFont));
            ArrayList<FontsModel> ret = new ArrayList<>();
            for (FontsData font : ordered)
                ret.add(new FontsModel(
                        font.getFont(),
                        font.getPartOf().size()-1
                        )
                );

            return ret;
        }

        public String getName() {
            return name;
        }

        public int getCount() {
            return count;
        }
    }
}
